@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -121,7 +121,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: FingerPrintResults.cc 34574 2015-06-03 13:01:29Z dmiller $ */
+/* $Id: FingerPrintResults.cc 35877 2016-06-15 20:15:11Z dmiller $ */
 
 #include "FingerPrintResults.h"
 #include "osscan.h"
@@ -252,6 +252,10 @@ const char *FingerPrintResultsIPv6::OmitSubmissionFP() {
     return reason;
   }
 
+  if (osscan_opentcpport <= 0 && osscan_closedtcpport <= 0) {
+    return "Missing a closed or open TCP port so results incomplete";
+  }
+
   if (incomplete) {
     return "Some probes failed to send so results incomplete";
   }
