@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -121,7 +121,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 34816 2015-07-02 04:14:51Z dmiller $ */
+/* $Id: NmapOps.cc 36155 2016-08-20 18:57:47Z abhishek $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -377,6 +377,7 @@ void NmapOps::Initialize() {
   scripttrace = 0;
   scriptupdatedb = 0;
   scripthelp = false;
+  scripttimeout = 0;
   chosenScripts.clear();
 #endif
   memset(&sourcesock, 0, sizeof(sourcesock));
@@ -570,8 +571,8 @@ dialog where you can start NPF if you have administrator privileges.";
     fatal("--min-rate=%g must be less than or equal to --max-rate=%g", min_packet_send_rate, max_packet_send_rate);
   }
 
-  if (af() == AF_INET6 && (generate_random_ips|numdecoys|bouncescan|fragscan)) {
-    fatal("Random targets, decoys, FTP bounce scan, and fragmentation are not supported with IPv6.");
+  if (af() == AF_INET6 && (generate_random_ips|bouncescan|fragscan)) {
+    fatal("Random targets, FTP bounce scan, and fragmentation are not supported with IPv6.");
   }
 
   if(ipoptions && osscan)
