@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -121,7 +121,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h 35347 2015-10-26 16:16:02Z dmiller $ */
+/* $Id: NmapOps.h 36155 2016-08-20 18:57:47Z abhishek $ */
 
 #ifndef NMAP_OPS_H
 #define NMAP_OPS_H
@@ -285,7 +285,7 @@ class NmapOps {
   /* Gets the spoofed MAC address, but returns NULL if it hasn't been set */
   const u8 *spoofMACAddress() { return spoof_mac_set? spoof_mac : NULL; }
 
-  int max_ips_to_scan; // Used for Random input (-iR) to specify how
+  unsigned int max_ips_to_scan; // Used for Random input (-iR) to specify how
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data-length op */
   char *extra_payload;
@@ -313,7 +313,7 @@ class NmapOps {
   int override_excludeports;
   int version_intensity;
 
-  struct in_addr decoys[MAX_DECOYS];
+  struct sockaddr_storage decoys[MAX_DECOYS];
   int osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
   int osscan_guess;   /* Be more aggressive in guessing OS type */
   int numdecoys;
@@ -390,6 +390,7 @@ class NmapOps {
   int scripttrace;
   int scriptupdatedb;
   bool scripthelp;
+  double scripttimeout;
   void chooseScripts(char* argument);
   std::vector<std::string> chosenScripts;
 #endif
@@ -401,8 +402,8 @@ class NmapOps {
   int ipopt_lasthop;	// offset in ipoptions where is space for targets ip for source/strict routing
 
   // Statistics Options set in nmap.cc
-  int numhosts_scanned;
-  int numhosts_up;
+  unsigned int numhosts_scanned;
+  unsigned int numhosts_up;
   int numhosts_scanning;
   stype current_scantype;
   bool noninteractive;
