@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -122,7 +122,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc 34798 2015-06-30 04:04:51Z dmiller $ */
+/* $Id: Target.cc 36124 2016-08-17 06:09:09Z abhishek $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
@@ -134,6 +134,7 @@
 #include "nbase.h"
 #include "NmapOps.h"
 #include "utils.h"
+#include "nmap.h"
 #include "nmap_error.h"
 
 extern NmapOps o;
@@ -352,12 +353,8 @@ void Target::setSourceSockAddr(const struct sockaddr_storage *ss, size_t ss_len)
 }
 
 // Returns IPv4 host address or {0} if unavailable.
-struct in_addr Target::v4source() const {
-  const struct in_addr *addy = v4sourceip();
-  struct in_addr in;
-  if (addy) return *addy;
-  in.s_addr = 0;
-  return in;
+struct sockaddr_storage Target::source() const {
+  return sourcesock;
 }
 
 // Returns IPv4 host address or NULL if unavailable.
@@ -429,8 +426,10 @@ const char *Target::NameIP(char *buf, size_t buflen) const {
 
 /* This next version returns a static buffer -- so no concurrency */
 const char *Target::NameIP() const {
-  if (!nameIPBuf) nameIPBuf = (char *) safe_malloc(MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
-  return NameIP(nameIPBuf, MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
+  /* Add 3 characters for the hostname and IP string, hence we allocate
+    (FQDN_LEN + INET6_ADDRSTRLEN + 4) octets, with octet for the null terminator */
+  if (!nameIPBuf) nameIPBuf = (char *) safe_malloc(FQDN_LEN + INET6_ADDRSTRLEN + 4);
+  return NameIP(nameIPBuf, FQDN_LEN + INET6_ADDRSTRLEN + 4);
 }
 
   /* Returns the next hop for sending packets to this host.  Returns true if
