@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -122,7 +122,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.h 34816 2015-07-02 04:14:51Z dmiller $ */
+/* $Id: Target.h 36124 2016-08-17 06:09:09Z abhishek $ */
 
 #ifndef TARGET_H
 #define TARGET_H
@@ -205,7 +205,7 @@ class Target {
   /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
   void setSourceSockAddr(const struct sockaddr_storage *ss, size_t ss_len);
-  struct in_addr v4source() const;
+  struct sockaddr_storage source() const;
   const struct in_addr *v4sourceip() const;
   const struct in6_addr *v6sourceip() const;
   /* The IPv4 or IPv6 literal string for the target host */
