@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -122,10 +122,21 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_winconfig.h 34574 2015-06-03 13:01:29Z dmiller $ */
+/* $Id: nbase_winconfig.h 35876 2016-06-15 13:56:03Z dmiller $ */
 
 #ifndef NBASE_WINCONFIG_H
 #define NBASE_WINCONFIG_H
+
+/* Define the earliest version of Windows we support.  These control
+what parts of the Windows API are available. The available constants
+are in <sdkddkver.h>.
+http://msdn.microsoft.com/en-us/library/aa383745.aspx
+http://blogs.msdn.com/oldnewthing/archive/2007/04/11/2079137.aspx */
+#undef _WIN32_WINNT
+#define _WIN32_WINNT _WIN32_WINNT_WIN6
+#undef NTDDI_VERSION
+#define NTDDI_VERSION NTDDI_WIN6
+
 //This disables the warning 4800 http://msdn.microsoft.com/en-us/library/b6801kcy(v=vs.71).aspx
 #pragma warning(disable : 4800)
 /* It doesn't really have struct IP, but we use a different one instead
@@ -152,6 +163,12 @@
 #define HAVE_SYS_STAT_H 1
 /* #define HAVE_INTTYPES_H */
 
+/* These functions are available on Vista and later */
+#if defined(_WIN32_WINNT) && _WIN32_WINNT >= _WIN32_WINNT_WIN6
+#define HAVE_INET_PTON 1
+#define HAVE_INET_NTOP 1
+#endif
+
 #ifdef _MSC_VER
 /* <wspiapi.h> only comes with Visual Studio. */
 #define HAVE_WSPIAPI_H 1
