@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -120,22 +120,13 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_winunix.h 34864 2015-07-08 11:58:57Z dmiller $ */
+/* $Id: nbase_winunix.h 35876 2016-06-15 13:56:03Z dmiller $ */
 
 #ifndef NBASE_WINUNIX_H
 #define NBASE_WINUNIX_H
 
-#include "nbase_winconfig.h"
 
-/* Define the earliest version of Windows we support.  These control
-   what parts of the Windows API are available. The available constants
-   are in <sdkddkver.h>.
-   http://msdn.microsoft.com/en-us/library/aa383745.aspx
-   http://blogs.msdn.com/oldnewthing/archive/2007/04/11/2079137.aspx */
-#undef _WIN32_WINNT
-#define _WIN32_WINNT _WIN32_WINNT_WIN2K
-#undef NTDDI_VERSION
-#define NTDDI_VERSION NTDDI_WIN2KSP4
+#include "nbase_winconfig.h"
 
 /* Winsock defines its own error codes that are analogous to but
    different from those in <errno.h>. The error macros have similar
