@@ -2,7 +2,7 @@
  * ncat_connect.c -- Ncat connect mode.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -118,7 +118,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_connect.c 34756 2015-06-27 08:21:53Z henri $ */
+/* $Id: ncat_connect.c 36004 2016-07-19 10:07:58Z abhishek $ */
 
 #include "base64.h"
 #include "nsock.h"
@@ -258,7 +258,7 @@ static void connect_report(nsock_iod nsi)
             char digest_buf[SHA1_STRING_LENGTH + 1];
             char *fp;
 
-            loguser("SSL connection to %s:%hu.", inet_socktop(&peer),
+            loguser("SSL connection to %s:%d.", inet_socktop(&peer),
                     nsock_iod_get_peerport(nsi));
 
             cert = SSL_get_peer_certificate((SSL *)nsock_iod_get_ssl(nsi));
@@ -285,7 +285,7 @@ static void connect_report(nsock_iod nsi)
                 loguser("Connected to %s.\n", peer.un.sun_path);
             else
 #endif
-                loguser("Connected to %s:%hu.\n", inet_socktop(&peer),
+                loguser("Connected to %s:%d.\n", inet_socktop(&peer),
                         nsock_iod_get_peerport(nsi));
         }
 #else
@@ -294,7 +294,7 @@ static void connect_report(nsock_iod nsi)
             loguser("Connected to %s.\n", peer.un.sun_path);
         else
 #endif
-            loguser("Connected to %s:%hu.\n", inet_socktop(&peer),
+            loguser("Connected to %s:%d.\n", inet_socktop(&peer),
                     nsock_iod_get_peerport(nsi));
 #endif
     }
@@ -909,9 +909,20 @@ int ncat_connect(void)
         {
             if (srcaddr.storage.ss_family != AF_UNIX) {
                 char *tmp_name = NULL;
+#if HAVE_MKSTEMP
+              char *tmpdir = getenv("TMPDIR");
+              size_t size=0, offset=0;
+              strbuf_sprintf(&tmp_name, &size, &offset, "%s/ncat.XXXXXX",
+                  tmpdir ? tmpdir : "/tmp");
+              if (mkstemp(tmp_name) == -1) {
+                bye("Failed to create name for temporary DGRAM source Unix domain socket (mkstemp).");
+              }
+              unlink(tmp_name);
+#else
                 /* If no source socket was specified, we have to create temporary one. */
                 if ((tmp_name = tempnam(NULL, "ncat.")) == NULL)
                     bye("Failed to create name for temporary DGRAM source Unix domain socket (tempnam).");
+#endif
 
                 srcaddr.un.sun_family = AF_UNIX;
                 strncpy(srcaddr.un.sun_path, tmp_name, sizeof(srcaddr.un.sun_path));
@@ -1067,6 +1078,13 @@ int ncat_connect(void)
     return rc == NSOCK_LOOP_ERROR ? 1 : 0;
 }
 
+static void send_udp_null(nsock_pool nsp)
+{
+  char *NULL_PROBE = "\0";
+  int length = 1;
+  nsock_write(nsp, cs.sock_nsi, write_socket_handler, -1, NULL, NULL_PROBE, length);
+}
+
 static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
 {
     enum nse_status status = nse_status(evt);
@@ -1075,10 +1093,12 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
     ncat_assert(type == NSE_TYPE_CONNECT || type == NSE_TYPE_CONNECT_SSL);
 
     if (status == NSE_STATUS_ERROR) {
-        loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
+        if (!o.zerobyte||o.verbose)
+          loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
         exit(1);
     } else if (status == NSE_STATUS_TIMEOUT) {
-        loguser("%s.\n", socket_strerror(ETIMEDOUT));
+        if (!o.zerobyte||o.verbose)
+          loguser("%s.\n", socket_strerror(ETIMEDOUT));
         exit(1);
     } else {
         ncat_assert(status == NSE_STATUS_SUCCESS);
@@ -1093,7 +1113,10 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
     }
 #endif
 
-    connect_report(cs.sock_nsi);
+    if (o.proto != IPPROTO_UDP && o.zerobyte) {
+      connect_report(cs.sock_nsi);
+      nsock_loop_quit(nsp);
+    }
 
     /* Create IOD for nsp->stdin */
     if ((cs.stdin_nsi = nsock_iod_new2(nsp, 0, NULL)) == NULL)
@@ -1123,12 +1146,15 @@ static void post_connect(nsock_pool nsp, nsock_iod iod)
 
     /* Start the initial reads. */
 
-    if (!o.sendonly)
+    if (!o.sendonly && !o.zerobyte)
         nsock_read(nsp, cs.sock_nsi, read_socket_handler, -1, NULL);
 
-    if (!o.recvonly)
+    if (!o.recvonly && !o.zerobyte)
         nsock_readbytes(nsp, cs.stdin_nsi, read_stdin_handler, -1, NULL, 0);
 
+    if (o.zerobyte && o.proto==IPPROTO_UDP)
+      send_udp_null(nsp);
+
     /* The --idle-timeout option says to exit after a certain period of
        inactivity. We start a timer here and reset it on every read event; see
        refresh_idle_timer. */
@@ -1147,6 +1173,7 @@ static void read_stdin_handler(nsock_pool nsp, nsock_event evt, void *data)
 
     ncat_assert(type == NSE_TYPE_READ);
 
+
     if (status == NSE_STATUS_EOF) {
         shutdown(nsock_iod_get_sd(cs.sock_nsi), SHUT_WR);
         /* In --send-only mode or non-TCP mode, exit after EOF on stdin. */
@@ -1201,7 +1228,8 @@ static void read_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
             nsock_loop_quit(nsp);
         return;
     } else if (status == NSE_STATUS_ERROR) {
-        loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
+        if (!o.zerobyte||o.verbose)
+          loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
         exit(1);
     } else if (status == NSE_STATUS_TIMEOUT) {
         loguser("%s.\n", socket_strerror(ETIMEDOUT));
@@ -1248,6 +1276,11 @@ static void write_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
         ncat_assert(status == NSE_STATUS_SUCCESS);
     }
 
+    if (o.zerobyte){
+      ncat_assert(o.proto == IPPROTO_UDP);
+      nsock_read(nsp, cs.sock_nsi, read_socket_handler, -1, NULL);
+      return;
+    }
     /* The write to the socket was successful. Allow reading more from stdin
        now. */
     nsock_readbytes(nsp, cs.stdin_nsi, read_stdin_handler, -1, NULL, 0);
@@ -1265,6 +1298,13 @@ static void idle_timer_handler(nsock_pool nsp, nsock_event evt, void *data)
 
     ncat_assert(status == NSE_STATUS_SUCCESS);
 
+    if (o.zerobyte&&o.proto==IPPROTO_UDP){
+      if (o.verbose)
+        loguser("UDP packet sent successfully\n");
+      nsock_loop_quit(nsp);
+      return;
+    }
+
     loguser("Idle timeout expired (%d ms).\n", o.idletimeout);
 
     exit(1);
