@@ -2,7 +2,7 @@
  * ncat_proxy.c -- HTTP proxy server.                                      *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -118,7 +118,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_proxy.c 34574 2015-06-03 13:01:29Z dmiller $ */
+/* $Id: ncat_proxy.c 35944 2016-07-01 11:44:26Z vincent $ */
 
 #include "base64.h"
 #include "http.h"
@@ -492,7 +492,7 @@ static int handle_connect(struct socket_buffer *client_sock,
         return 400;
     }
     if (o.debug > 1)
-        logdebug("CONNECT to %s:%hu.\n", request->uri.host, request->uri.port);
+        logdebug("CONNECT to %s:%d.\n", request->uri.host, request->uri.port);
 
     rc = resolve(request->uri.host, request->uri.port, &su.storage, &sslen, o.af);
     if (rc != 0) {
@@ -680,7 +680,7 @@ static int do_transaction(struct http_request *request,
     if (request->uri.port == -1 || request->uri.port == 80)
         n = Snprintf(buf, sizeof(buf), "%s", request->uri.host);
     else
-        n = Snprintf(buf, sizeof(buf), "%s:%hu", request->uri.host, request->uri.port);
+        n = Snprintf(buf, sizeof(buf), "%s:%d", request->uri.host, request->uri.port);
     if (n < 0 || n >= sizeof(buf)) {
         /* Request Entity Too Large. */
         return 501;
