@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -120,13 +120,15 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_ftp.cc 34574 2015-06-03 13:01:29Z dmiller $ */
+/* $Id: nmap_ftp.cc 36049 2016-07-29 15:50:49Z dmiller $ */
+#include "nmap.h"
 #include "nmap_ftp.h"
 #include "output.h"
 #include "NmapOps.h"
 #include "nmap_error.h"
 #include "tcpip.h"
 #include "Target.h"
+#include "nmap_tty.h"
 extern NmapOps o;
 
 struct ftpinfo get_default_ftpinfo(void) {
@@ -166,9 +168,9 @@ int parse_bounce_argument(struct ftpinfo *ftp, char *url) {
     ftp->port = atoi(s);
   }
 
-  strncpy(ftp->server_name, q, MAXHOSTNAMELEN);
+  strncpy(ftp->server_name, q, FQDN_LEN+1);
 
-  ftp->user[63] = ftp->pass[255] = ftp->server_name[MAXHOSTNAMELEN] = 0;
+  ftp->user[63] = ftp->pass[255] = ftp->server_name[FQDN_LEN] = 0;
 
   return 1;
 }
@@ -219,6 +221,14 @@ int ftp_anon_connect(struct ftpinfo *ftp) {
   if (recvbuf[0] == '5')
     fatal("Your FTP bounce server doesn't like the username \"%s\"", ftp->user);
 
+  if (!strncmp(recvbuf, "230", 3)) {
+    // 230 User logged in
+    // No need to send PASS
+    if (o.verbose)
+      log_write(LOG_STDOUT, "Login credentials accepted by FTP server!\n");
+    ftp->sd = sd;
+    return sd;
+  }
   Snprintf(command, 511, "PASS %s\r\n", ftp->pass);
 
   send(sd, command, strlen(command), 0);
@@ -256,33 +266,32 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
                  struct ftpinfo *ftp) {
   o.current_scantype = BOUNCE_SCAN;
 
-  time_t starttime;
+  ScanProgressMeter *SPM;
   int res , sd = ftp->sd,  i = 0;
   const char *t = (const char *)target->v4hostip();
   int retriesleft = FTP_RETRIES;
   char recvbuf[2048];
   char targetstr[20];
   char command[512];
-  char hostname[1200];
   unsigned short portno, p1, p2;
   int timedout;
+  bool privok = false;
 
   if (numports == 0)
     return; /* nothing to scan for */
 
   Snprintf(targetstr, 20, "%d,%d,%d,%d,", UC(t[0]), UC(t[1]), UC(t[2]), UC(t[3]));
 
-  starttime = time(NULL);
-  if (o.verbose || o.debugging) {
-    struct tm *tm = localtime(&starttime);
-    assert(tm);
-    log_write(LOG_STDOUT, "Initiating TCP FTP bounce scan against %s at %02d:%02d\n", target->NameIP(hostname, sizeof(hostname)), tm->tm_hour, tm->tm_min );
-  }
+  SPM = new ScanProgressMeter(scantype2str(BOUNCE_SCAN));
   for (i = 0; i < numports; i++) {
 
     /* Check for timeout */
-    if (target->timedOut(NULL))
+    if (target->timedOut(NULL)) {
+      Snprintf(recvbuf, sizeof(recvbuf), "Target timed out");
+      SPM->endTask(NULL, recvbuf);
+      delete SPM;
       return;
+    }
 
     portno = htons(portarray[i]);
     p1 = ((unsigned char *) &portno)[0];
@@ -298,14 +307,21 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
         retriesleft--;
         close(sd);
         ftp->sd = ftp_anon_connect(ftp);
-        if (ftp->sd < 0)
+        if (ftp->sd < 0) {
+          Snprintf(recvbuf, sizeof(recvbuf), "Error connecting");
+          SPM->endTask(NULL, recvbuf);
+          delete SPM;
           return;
+        }
         sd = ftp->sd;
         i--;
       } else {
         error("Our socket descriptor is dead and we are out of retries. Giving up.");
         close(sd);
         ftp->sd = -1;
+        Snprintf(recvbuf, sizeof(recvbuf), "Max retries exceeded");
+        SPM->endTask(NULL, recvbuf);
+        delete SPM;
         return;
       }
     } else { /* Our send is good */
@@ -317,22 +333,26 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
         if (o.debugging)
           log_write(LOG_STDOUT, "result of port query on port %i: %s",
                                      portarray[i],  recvbuf);
-        if (recvbuf[0] == '5') {
+        if (recvbuf[0] == '5' && !privok) {
           if (portarray[i] > 1023) {
             fatal("Your FTP bounce server sucks, it won't let us feed bogus ports!");
           } else {
             error("Your FTP bounce server doesn't allow privileged ports, skipping them.");
             while (i < numports && portarray[i] < 1024) i++;
-            if (!portarray[i]) {
+            if (i >= numports) {
               fatal("And you didn't want to scan any unprivileged ports.  Giving up.");
             }
           }
         } else { /* Not an error message */
+          if (portarray[i] < 1024) {
+            privok = true;
+          }
           if (send(sd, "LIST\r\n", 6, 0) > 0 ) {
             res = recvtime(sd, recvbuf, 2048, 12, &timedout);
             if (res < 0) {
               perror("recv problem from FTP bounce server");
             } else if (res == 0) {
+              recvbuf[res] = '\0';
               if (timedout)
                 target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_FILTERED);
               else target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_CLOSED);
@@ -346,25 +366,35 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
                   error("FTP command misalignment detected ... correcting.");
                 res = recvtime(sd, recvbuf, 2048, 10, NULL);
               }
-              if (recvbuf[0] == '1' || recvbuf[0] == '2') {
-                target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_OPEN);
-                if (recvbuf[0] == '1') {
-                  res = recvtime(sd, recvbuf, 2048, 5, NULL);
-                  if (res < 0)
-                    perror("recv problem from FTP bounce server");
-                  else {
-                    recvbuf[res] = '\0';
-                    if (res > 0) {
-                      if (o.debugging)
-                        log_write(LOG_STDOUT, "nxt line: %s", recvbuf);
-                      if (recvbuf[0] == '4' && recvbuf[1] == '2' && recvbuf[2] == '6') {
-                        target->ports.forgetPort(portarray[i], IPPROTO_TCP);
-                        if (o.debugging || o.verbose)
-                          log_write(LOG_STDOUT, "Changed my mind about port %i\n", portarray[i]);
-                      }
+              if (recvbuf[0] == '1') {
+                res = recvtime(sd, recvbuf, 2048, 10, &timedout);
+                if (res < 0)
+                  perror("recv problem from FTP bounce server");
+                else if (timedout || res == 0) {
+                  // Timed out waiting for LIST to complete; probably filtered.
+                  if(send(sd, "ABOR\r\n", 6, 0) > 0) {
+                    target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_FILTERED);
+                  }
+                  // Get response and discard
+                  res = recvtime(sd, recvbuf, 2048, 10, &timedout);
+                  recvbuf[0] = '\0';
+                  goto nextport;
+                }
+                else {
+                  recvbuf[res] = '\0';
+                  if (res > 0) {
+                    if (o.debugging)
+                      log_write(LOG_STDOUT, "nxt line: %s", recvbuf);
+                    if (recvbuf[0] == '4' && recvbuf[1] == '2' && recvbuf[2] == '6') {
+                      target->ports.forgetPort(portarray[i], IPPROTO_TCP);
+                      if (o.debugging || o.verbose)
+                        log_write(LOG_STDOUT, "Changed my mind about port %i\n", portarray[i]);
                     }
                   }
                 }
+              }
+              if (recvbuf[0] == '2') {
+                target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_OPEN);
               } else {
                 /* This means the port is closed ... */
                 target->ports.setPortState(portarray[i], IPPROTO_TCP, PORT_CLOSED);
@@ -374,10 +404,18 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
         }
       }
     }
+    nextport:
+    if (SPM->mayBePrinted(NULL)) {
+      SPM->printStatsIfNecessary((double) i / numports, NULL);
+    }
+    else if (keyWasPressed()) {
+      SPM->printStats((double) i / numports, NULL);
+      log_flush(LOG_STDOUT);
+    }
   }
 
-  if (o.debugging || o.verbose)
-    log_write(LOG_STDOUT, "Scanned %d ports in %ld seconds via the Bounce scan.\n",
-              numports, (long) time(NULL) - starttime);
+  Snprintf(recvbuf, sizeof(recvbuf), "%d total ports", numports);
+  SPM->endTask(NULL, recvbuf);
+  delete SPM;
   return;
 }
