@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -124,14 +124,29 @@
 
 #include "nbase.h"
 
+#ifdef HAVE_CONFIG_H
+#include "nmap_config.h"
+#endif
+
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <string.h>
+#include <openssl/bn.h>
 #include <openssl/bio.h>
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
+#include <openssl/x509v3.h>
+#include <openssl/evp.h>
+#include <openssl/err.h>
+
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && !defined LIBRESSL_VERSION_NUMBER
+/* Technically some of these things were added in 0x10100006
+ * but that was pre-release. */
+#define HAVE_OPAQUE_STRUCTS 1
+#endif
+
 
 extern "C"
 {
@@ -146,6 +161,12 @@ struct cert_userdata {
   int attributes_table;
 };
 
+/* from nse_openssl.cc */
+typedef struct bignum_data {
+  BIGNUM * bn;
+} bignum_data_t;
+
+
 SSL *nse_nsock_get_ssl(lua_State *L);
 
 /* This is a reference to a table that will be used as the metatable for
@@ -251,6 +272,54 @@ static void x509_name_to_table(lua_State *L, X509_NAME *name)
   }
 }
 
+static bool x509_extensions_to_table(lua_State *L, const STACK_OF(X509_EXTENSION) *exts)
+{
+  if (sk_X509_EXTENSION_num(exts) <= 0)
+    return false;
+
+  lua_newtable(L);
+
+  for (int i = 0; i < sk_X509_EXTENSION_num(exts); i++) {
+    ASN1_OBJECT *obj;
+    X509_EXTENSION *ext;
+    char *value = NULL;
+    BIO *out;
+
+    ext = sk_X509_EXTENSION_value(exts, i);
+    obj = X509_EXTENSION_get_object(ext);
+
+    lua_newtable(L);
+    char objname[256];
+    long len = 0;
+    len = OBJ_obj2txt(objname, 256, obj, 0);
+    lua_pushlstring(L, objname, MIN(len, 256));
+    lua_setfield(L, -2, "name");
+
+
+    if (X509_EXTENSION_get_critical(ext)) {
+      lua_pushboolean(L, true);
+      lua_setfield(L, -2, "critical");
+    }
+
+    out = BIO_new(BIO_s_mem());
+    if (!X509V3_EXT_print(out, ext, 0, 0)) {
+      lua_pushboolean(L, true);
+      lua_setfield(L, -2, "error");
+    }
+    else {
+      len = BIO_get_mem_data(out, &value);
+      lua_pushlstring(L, value, len);
+      lua_setfield(L, -2, "value");
+    }
+    BIO_free_all(out);
+
+    lua_seti(L, -2, i+1);
+  }
+
+  return true;
+
+}
+
 /* Parse as a decimal integer the len characters starting at s. This function
    can only process positive numbers; if the return value is negative then a
    parsing error occurred. */
@@ -377,7 +446,7 @@ static void asn1_time_to_obj(lua_State *L, const ASN1_TIME *s)
 /* This is a helper function for x509_validity_to_table. It builds a table with
    the two members "notBefore" and "notAfter", whose values are what is returned
    from asn1_time_to_obj. */
-static void x509_validity_to_table(lua_State *L, const X509 *cert)
+static void x509_validity_to_table(lua_State *L, X509 *cert)
 {
   lua_newtable(L);
 
@@ -417,7 +486,7 @@ static const char *pkey_type_to_string(int type)
     return "dsa";
   case EVP_PKEY_DH:
     return "dh";
-#ifdef EVP_PKEY_EC
+#ifdef HAVE_OPENSSL_EC
   case EVP_PKEY_EC:
     return "ec";
 #endif
@@ -426,20 +495,19 @@ static const char *pkey_type_to_string(int type)
   }
 }
 
-void lua_push_ecdhparams(lua_State *L, EVP_PKEY *pubkey) {
-#ifdef EC_KEY
+int lua_push_ecdhparams(lua_State *L, EVP_PKEY *pubkey) {
+#ifdef HAVE_OPENSSL_EC
   EC_KEY *ec_key = EVP_PKEY_get1_EC_KEY(pubkey);
   const EC_GROUP *group = EC_KEY_get0_group(ec_key);
   int nid;
+  /* This structure (ecdhparams.curve_params) comes from tls.lua */
+  lua_newtable(L); /* ecdhparams */
+  lua_newtable(L); /* curve_params */
   if ((nid = EC_GROUP_get_curve_name(group)) != 0) {
-    lua_newtable(L);
-    lua_newtable(L);
     lua_pushstring(L, OBJ_nid2sn(nid));
     lua_setfield(L, -2, "curve");
     lua_pushstring(L, "namedcurve");
     lua_setfield(L, -2, "ec_curve_type");
-    lua_setfield(L, -2, "curve_params");
-    lua_setfield(L, -2, "ecdhparams");
   }
   else {
     /* According to RFC 5480 section 2.1.1, explicit curves must not be used with
@@ -447,26 +515,22 @@ void lua_push_ecdhparams(lua_State *L, EVP_PKEY *pubkey) {
        to add in code to extract the extra parameters. */
     nid = EC_METHOD_get_field_type(EC_GROUP_method_of(group));
     if (nid == NID_X9_62_prime_field) {
-      lua_newtable(L);
-      lua_newtable(L);
       lua_pushstring(L, "explicit_prime");
-      lua_setfield(L, -2, "ec_curve_type");
-      lua_setfield(L, -2, "curve_params");
-      lua_setfield(L, -2, "ecdhparams");
     }
     else if (nid == NID_X9_62_characteristic_two_field) {
-      lua_newtable(L);
-      lua_newtable(L);
       lua_pushstring(L, "explicit_char2");
-      lua_setfield(L, -2, "ec_curve_type");
-      lua_setfield(L, -2, "curve_params");
-      lua_setfield(L, -2, "ecdhparams");
     }
     else {
-      /* Something wierd happened. */
+      /* Something weird happened. */
+      lua_pushstring(L, "UNKNOWN");
     }
+    lua_setfield(L, -2, "ec_curve_type");
   }
+  lua_setfield(L, -2, "curve_params");
   EC_KEY_free(ec_key);
+  return 1;
+#else
+  return 0;
 #endif
 }
 
@@ -524,7 +588,11 @@ static int parse_ssl_cert(lua_State *L, X509 *cert)
     lua_setfield(L, -2, "subject");
   }
 
+#if HAVE_OPAQUE_STRUCTS
+  const char *sig_algo = OBJ_nid2ln(X509_get_signature_nid(cert));
+#else
   const char *sig_algo = OBJ_nid2ln(OBJ_obj2nid(cert->sig_alg->algorithm));
+#endif
   lua_pushstring(L, sig_algo);
   lua_setfield(L, -2, "sig_algorithm");
 
@@ -540,15 +608,50 @@ static int parse_ssl_cert(lua_State *L, X509 *cert)
   cert_pem_to_string(L, cert);
   lua_setfield(L, -2, "pem");
 
+#if HAVE_OPAQUE_STRUCTS
+  if (x509_extensions_to_table(L, X509_get0_extensions(cert))) {
+#else
+  if (x509_extensions_to_table(L, cert->cert_info->extensions)) {
+#endif
+    lua_setfield(L, -2, "extensions");
+  }
+
   pubkey = X509_get_pubkey(cert);
+  if (pubkey == NULL) {
+    lua_pushnil(L);
+    lua_pushfstring(L, "Error parsing cert: %s", ERR_error_string(ERR_get_error(), NULL));
+    return 2;
+  }
   lua_newtable(L);
+#if HAVE_OPAQUE_STRUCTS
+  pkey_type = EVP_PKEY_base_id(pubkey);
+#else
   pkey_type = EVP_PKEY_type(pubkey->type);
+#endif
+#ifdef HAVE_OPENSSL_EC
   if (pkey_type == EVP_PKEY_EC) {
     lua_push_ecdhparams(L, pubkey);
+    lua_setfield(L, -2, "ecdhparams");
+  }
+  else
+#endif
+  if (pkey_type == EVP_PKEY_RSA) {
+    RSA *rsa = EVP_PKEY_get1_RSA(pubkey);
+    bignum_data_t * data = (bignum_data_t *) lua_newuserdata( L, sizeof(bignum_data_t));
+    luaL_getmetatable( L, "BIGNUM" );
+    lua_setmetatable( L, -2 );
+  #if HAVE_OPAQUE_STRUCTS
+    const BIGNUM *n, *e, *d;
+    RSA_get0_key(rsa, &n, &e, &d);
+    data->bn = (BIGNUM*) e;
+  #else
+    data->bn = rsa->e;
+  #endif
+    lua_setfield(L, -2, "exponent");
   }
   lua_pushstring(L, pkey_type_to_string(pkey_type));
   lua_setfield(L, -2, "type");
-  lua_pushnumber(L, EVP_PKEY_bits(pubkey));
+  lua_pushinteger(L, EVP_PKEY_bits(pubkey));
   lua_setfield(L, -2, "bits");
   lua_setfield(L, -2, "pubkey");
   EVP_PKEY_free(pubkey);
