@@ -121,7 +121,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: osscan.h 35761 2016-04-04 15:38:44Z dmiller $ */
+/* $Id: osscan.h 36347 2016-09-30 04:15:44Z dmiller $ */
 
 #ifndef OSSCAN_H
 #define OSSCAN_H
@@ -172,6 +172,9 @@ struct OS_Classification {
    classifications. */
 struct FingerMatch {
   int line; /* For reference prints, the line # in nmap-os-db */
+  /* For IPv6 matches, the number of fingerprints that contributed to this
+   * classification group */
+  unsigned short numprints;
   char *OS_name;
   std::vector<OS_Classification> OS_class;
 
