@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -122,7 +122,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.cc 34806 2015-06-30 18:34:28Z dmiller $ */
+/* $Id: tcpip.cc 36181 2016-08-25 18:41:03Z dmiller $ */
 
 #include "nmap.h"
 
@@ -146,18 +146,8 @@
 
 extern NmapOps o;
 
-#ifdef WIN32
-#include "pcap-int.h"
-#endif
-
 static PacketCounter PktCt;
 
-
-
-
-
-
-
 /* Create a raw socket and do things that always apply to raw sockets:
     * Set SO_BROADCAST.
     * Set IP_HDRINCL.
@@ -235,11 +225,11 @@ void PacketTrace::traceArp(pdirection pdir, const u8 *frame, u32 len,
   }
 
   if (frame[7] == 1) { /* arp REQUEST */
-    inet_ntop(AF_INET, frame + 24, who_has, sizeof(who_has));
-    inet_ntop(AF_INET, frame + 14, tell, sizeof(tell));
+    inet_ntop(AF_INET, (void *)(frame + 24), who_has, sizeof(who_has));
+    inet_ntop(AF_INET, (void *)(frame + 14), tell, sizeof(tell));
     Snprintf(arpdesc, sizeof(arpdesc), "who-has %s tell %s", who_has, tell);
   } else { /* ARP REPLY */
-    inet_ntop(AF_INET, frame + 14, who_has, sizeof(who_has));
+    inet_ntop(AF_INET, (void *)(frame + 14), who_has, sizeof(who_has));
     Snprintf(arpdesc, sizeof(arpdesc),
              "reply %s is-at %02X:%02X:%02X:%02X:%02X:%02X", who_has,
              frame[8], frame[9], frame[10], frame[11], frame[12],
@@ -839,7 +829,7 @@ int send_tcp_raw_decoys(int sd, const struct eth_nfo *eth,
 
   for (decoy = 0; decoy < o.numdecoys; decoy++)
     if (send_tcp_raw(sd, eth,
-                     &o.decoys[decoy], victim,
+                     &((struct sockaddr_in *)&o.decoys[decoy])->sin_addr, victim,
                      ttl, df,
                      ipopt, ipoptlen,
                      sport, dport,
@@ -956,7 +946,7 @@ int send_udp_raw_decoys(int sd, const struct eth_nfo *eth,
   int decoy;
 
   for (decoy = 0; decoy < o.numdecoys; decoy++)
-    if (send_udp_raw(sd, eth, &o.decoys[decoy], victim,
+    if (send_udp_raw(sd, eth, &((struct sockaddr_in *)&o.decoys[decoy])->sin_addr, victim,
                      ttl, ipid, ipops, ipoptlen,
                      sport, dport, data, datalen) == -1)
       return -1;
@@ -1269,7 +1259,7 @@ int readtcppacket(const u8 *packet, int readdata) {
         log_write(LOG_PLAIN, "URG ");
       log_write(LOG_PLAIN, "\n");
 
-      log_write(LOG_PLAIN, "ipid: %hu ttl: %hu ", ntohs(ip->ip_id),
+      log_write(LOG_PLAIN, "ipid: %hu ttl: %hhu ", ntohs(ip->ip_id),
                 ip->ip_ttl);
 
       if (tcp->th_flags & (TH_SYN | TH_ACK))
@@ -1329,7 +1319,7 @@ int readudppacket(const u8 *packet, int readdata) {
                 sourcehost, ntohs(udp->uh_sport), inet_ntoa(bullshit2),
                 ntohs(udp->uh_dport), tot_len);
 
-      log_write(LOG_PLAIN, "ttl: %hu ", ip->ip_ttl);
+      log_write(LOG_PLAIN, "ttl: %hhu ", ip->ip_ttl);
     }
   }
   if (readdata && i < tot_len) {
@@ -1669,18 +1659,6 @@ char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec,
   }
 
   do {
-#ifdef WIN32
-    long to_left;
-
-    if (to_usec > 0) {
-      gettimeofday(&tv_end, NULL);
-      to_left = MAX(1, (to_usec - TIMEVAL_SUBTRACT(tv_end, tv_start)) / 1000);
-    } else {
-      to_left = 1;
-    }
-    // Set the timeout (BUGBUG: this is cheating)
-    PacketSetReadTimeout(pd->adapter, to_left);
-#endif
 
     p = NULL;
     /* It may be that protecting this with !pcap_selectable_fd_one_to_one is not
