@@ -123,7 +123,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.cc 36172 2016-08-22 19:50:06Z dmiller $ */
+/* $Id: timing.cc 36363 2016-10-16 13:58:10Z dmiller $ */
 
 #include "timing.h"
 #include "NmapOps.h"
@@ -131,6 +131,7 @@
 #include "xml.h"
 
 #include <math.h>
+#include <limits>
 
 extern NmapOps o;
 
@@ -437,7 +438,7 @@ void RateMeter::update(double amount, const struct timeval *now) {
     interval = MAX(current_rate_history, diff);
   else
     interval = TIMEVAL_SUBTRACT(*now, start_tv) / 1000000.0;
-  assert(diff <= interval);
+  assert(diff <= interval + std::numeric_limits<double>::epsilon());
   /* If we record an amount in the very same instant that the timer is started,
      there's no way to calculate meaningful rates. Ignore it. */
   if (interval == 0.0)
